# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .cost_details import CostDetails
from .shared.pay_i_common_models_budget_management_cost_details_base import (
    PayICommonModelsBudgetManagementCostDetailsBase,
)

__all__ = ["IngestResponse", "XproxyResult", "XproxyResultCost", "XproxyResultLimits"]


class XproxyResultCost(BaseModel):
    currency: Optional[Literal["usd"]] = None

    input: Optional[PayICommonModelsBudgetManagementCostDetailsBase] = None

    output: Optional[PayICommonModelsBudgetManagementCostDetailsBase] = None

    total: Optional[CostDetails] = None


class XproxyResultLimits(BaseModel):
    state: Optional[Literal["ok", "blocked", "blocked_external", "exceeded", "overrun", "failed"]] = None


class XproxyResult(BaseModel):
    blocked_limit_ids: Optional[List[str]] = None

    cost: Optional[XproxyResultCost] = None

    experience_id: Optional[str] = None

    limits: Optional[Dict[str, XproxyResultLimits]] = None

    request_id: Optional[str] = None

    request_tags: Optional[List[str]] = None

    resource_id: Optional[str] = None

    user_id: Optional[str] = None


class IngestResponse(BaseModel):
    event_timestamp: datetime

    ingest_timestamp: datetime

    request_id: str

    xproxy_result: XproxyResult



import logging
from typing import Any, Union

import tiktoken
from wrapt import wrap_function_wrapper  # type: ignore

from payi.types import IngestUnitsParams
from payi.types.ingest_units_params import Units

from .instrument import IsStreaming, PayiInstrumentor


class AnthropicIntrumentor:
    @staticmethod
    def instrument(instrumentor: PayiInstrumentor) -> None:
        try:
            import anthropic  # type: ignore #  noqa: F401  I001

            wrap_function_wrapper(
                "anthropic.resources.messages",
                "Messages.create",
                chat_wrapper(instrumentor),
            )

            wrap_function_wrapper(
                "anthropic.resources.messages",
                "Messages.stream",
                chat_wrapper(instrumentor),
            )

            wrap_function_wrapper(
                "anthropic.resources.messages",
                "AsyncMessages.create",
                achat_wrapper(instrumentor),
            )

            wrap_function_wrapper(
                "anthropic.resources.messages",
                "AsyncMessages.stream",
                achat_wrapper(instrumentor),
            )

        except Exception as e:
            logging.debug(f"Error instrumenting anthropic: {e}")
            return


@PayiInstrumentor.payi_wrapper
def chat_wrapper(
    instrumentor: PayiInstrumentor,
    wrapped: Any,
    instance: Any,
    *args: Any,
    **kwargs: Any,
) -> Any:
    return instrumentor.chat_wrapper(
        "system.anthropic",
        process_chunk,
        process_request,
        process_synchronous_response,
        IsStreaming.kwargs,
        wrapped,
        instance,
        args,
        kwargs,
    )

@PayiInstrumentor.payi_awrapper
async def achat_wrapper(
    instrumentor: PayiInstrumentor,
    wrapped: Any,
    instance: Any,
    *args: Any,
    **kwargs: Any,
) -> Any:
    return await instrumentor.achat_wrapper(
        "system.anthropic",
        process_chunk,
        process_request,
        process_synchronous_response,
        IsStreaming.kwargs,
        wrapped,
        instance,
        args,
        kwargs,
    )


def process_chunk(chunk: Any, ingest: IngestUnitsParams) -> None:
    if chunk.type == "message_start":
        usage = chunk.message.usage
        units = ingest["units"]

        input = PayiInstrumentor.update_for_vision(usage.input_tokens, units)

        units["text"] = Units(input=input, output=0)

        if hasattr(usage, "cache_creation_input_tokens") and usage.cache_creation_input_tokens > 0:
            text_cache_write = usage.cache_creation_input_tokens
            units["text_cache_write"] = Units(input=text_cache_write, output=0)

        if hasattr(usage, "cache_read_input_tokens") and usage.cache_read_input_tokens > 0:
            text_cache_read = usage.cache_read_input_tokens
            units["text_cache_read"] = Units(input=text_cache_read, output=0)

    elif chunk.type == "message_delta":
        usage = chunk.usage
        ingest["units"]["text"]["output"] = usage.output_tokens


def process_synchronous_response(response: Any, ingest: IngestUnitsParams, log_prompt_and_response: bool, *args: Any, **kwargs: 'dict[str, Any]') -> Any: # noqa: ARG001
    usage = response.usage
    input = usage.input_tokens
    output = usage.output_tokens
    units: dict[str, Units] = ingest["units"]

    if hasattr(usage, "cache_creation_input_tokens") and usage.cache_creation_input_tokens > 0:
        text_cache_write = usage.cache_creation_input_tokens
        units["text_cache_write"] = Units(input=text_cache_write, output=0)

    if hasattr(usage, "cache_read_input_tokens") and usage.cache_read_input_tokens > 0:
        text_cache_read = usage.cache_read_input_tokens
        units["text_cache_read"] = Units(input=text_cache_read, output=0)

    input = PayiInstrumentor.update_for_vision(input, units)

    units["text"] = Units(input=input, output=output)

    if log_prompt_and_response:
        ingest["provider_response_json"] = response.to_json()

    return None

def has_image_and_get_texts(encoding: tiktoken.Encoding, content: Union[str, 'list[Any]']) -> 'tuple[bool, int]':
    if isinstance(content, str):
        return False, 0
    elif isinstance(content, list): # type: ignore
        has_image = any(item.get("type") == "image" for item in content)
        if has_image is False:
            return has_image, 0
        
        token_count = sum(len(encoding.encode(item.get("text", ""))) for item in content if item.get("type") == "text")
        return has_image, token_count

def process_request(ingest: IngestUnitsParams, *args: Any, **kwargs: Any) -> None: # noqa: ARG001
    messages = kwargs.get("messages")
    if not messages or len(messages) == 0:
        return
    
    estimated_token_count = 0 
    has_image = False

    enc = tiktoken.get_encoding("cl100k_base")
    
    for message in messages:
        msg_has_image, msg_prompt_tokens = has_image_and_get_texts(enc, message.get('content', ''))
        if msg_has_image:
            has_image = True
            estimated_token_count += msg_prompt_tokens
    
    if not has_image or estimated_token_count == 0:
        return

    ingest["units"][PayiInstrumentor.estimated_prompt_tokens] = Units(input=estimated_token_count, output=0)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .price_modifier import PriceModifier

__all__ = ["PriceModifierRetrieveResponse"]

PriceModifierRetrieveResponse: TypeAlias = List[PriceModifier]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from tests.utils import assert_matches_type
from payi.types.experiences import (
    ExperienceType,
    TypeListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTypes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Payi) -> None:
        type = client.experiences.types.create(
            description="x",
            name="x",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Payi) -> None:
        type = client.experiences.types.create(
            description="x",
            name="x",
            limit_config={
                "max": 0,
                "limit_tags": ["tag1", "tag2"],
                "limit_type": "block",
                "threshold": 0,
            },
            logging_enabled=True,
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Payi) -> None:
        response = client.experiences.types.with_raw_response.create(
            description="x",
            name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Payi) -> None:
        with client.experiences.types.with_streaming_response.create(
            description="x",
            name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Payi) -> None:
        type = client.experiences.types.retrieve(
            "experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Payi) -> None:
        response = client.experiences.types.with_raw_response.retrieve(
            "experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Payi) -> None:
        with client.experiences.types.with_streaming_response.retrieve(
            "experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            client.experiences.types.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Payi) -> None:
        type = client.experiences.types.update(
            experience_name="experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Payi) -> None:
        type = client.experiences.types.update(
            experience_name="experience_name",
            description="description",
            logging_enabled=True,
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Payi) -> None:
        response = client.experiences.types.with_raw_response.update(
            experience_name="experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Payi) -> None:
        with client.experiences.types.with_streaming_response.update(
            experience_name="experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            client.experiences.types.with_raw_response.update(
                experience_name="",
            )

    @parametrize
    def test_method_list(self, client: Payi) -> None:
        type = client.experiences.types.list()
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Payi) -> None:
        type = client.experiences.types.list(
            name="name",
        )
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Payi) -> None:
        response = client.experiences.types.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = response.parse()
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Payi) -> None:
        with client.experiences.types.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = response.parse()
            assert_matches_type(TypeListResponse, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Payi) -> None:
        type = client.experiences.types.delete(
            "experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Payi) -> None:
        response = client.experiences.types.with_raw_response.delete(
            "experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Payi) -> None:
        with client.experiences.types.with_streaming_response.delete(
            "experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            client.experiences.types.with_raw_response.delete(
                "",
            )


class TestAsyncTypes:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.create(
            description="x",
            name="x",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.create(
            description="x",
            name="x",
            limit_config={
                "max": 0,
                "limit_tags": ["tag1", "tag2"],
                "limit_type": "block",
                "threshold": 0,
            },
            logging_enabled=True,
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPayi) -> None:
        response = await async_client.experiences.types.with_raw_response.create(
            description="x",
            name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = await response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPayi) -> None:
        async with async_client.experiences.types.with_streaming_response.create(
            description="x",
            name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = await response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.retrieve(
            "experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPayi) -> None:
        response = await async_client.experiences.types.with_raw_response.retrieve(
            "experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = await response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPayi) -> None:
        async with async_client.experiences.types.with_streaming_response.retrieve(
            "experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = await response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            await async_client.experiences.types.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.update(
            experience_name="experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.update(
            experience_name="experience_name",
            description="description",
            logging_enabled=True,
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPayi) -> None:
        response = await async_client.experiences.types.with_raw_response.update(
            experience_name="experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = await response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPayi) -> None:
        async with async_client.experiences.types.with_streaming_response.update(
            experience_name="experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = await response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            await async_client.experiences.types.with_raw_response.update(
                experience_name="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.list()
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.list(
            name="name",
        )
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPayi) -> None:
        response = await async_client.experiences.types.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = await response.parse()
        assert_matches_type(TypeListResponse, type, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPayi) -> None:
        async with async_client.experiences.types.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = await response.parse()
            assert_matches_type(TypeListResponse, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncPayi) -> None:
        type = await async_client.experiences.types.delete(
            "experience_name",
        )
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncPayi) -> None:
        response = await async_client.experiences.types.with_raw_response.delete(
            "experience_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        type = await response.parse()
        assert_matches_type(ExperienceType, type, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncPayi) -> None:
        async with async_client.experiences.types.with_streaming_response.delete(
            "experience_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            type = await response.parse()
            assert_matches_type(ExperienceType, type, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `experience_name` but received ''"):
            await async_client.experiences.types.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.limits import tag_create_params, tag_remove_params, tag_update_params
from ...types.limits.tag_list_response import TagListResponse
from ...types.limits.tag_create_response import TagCreateResponse
from ...types.limits.tag_delete_response import TagDeleteResponse
from ...types.limits.tag_remove_response import TagRemoveResponse
from ...types.limits.tag_update_response import TagUpdateResponse

__all__ = ["TagsResource", "AsyncTagsResource"]


class TagsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return TagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return TagsResourceWithStreamingResponse(self)

    def create(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagCreateResponse:
        """
        Add Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return self._post(
            f"/api/v1/limits/{limit_id}/tags",
            body=maybe_transform({"limit_tags": limit_tags}, tag_create_params.TagCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagCreateResponse,
        )

    def update(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagUpdateResponse:
        """
        Update the Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return self._put(
            f"/api/v1/limits/{limit_id}/tags",
            body=maybe_transform({"limit_tags": limit_tags}, tag_update_params.TagUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagUpdateResponse,
        )

    def list(
        self,
        limit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagListResponse:
        """
        Get Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return self._get(
            f"/api/v1/limits/{limit_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )

    def delete(
        self,
        limit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagDeleteResponse:
        """
        Delete all Tags from Limit

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return self._delete(
            f"/api/v1/limits/{limit_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagDeleteResponse,
        )

    def remove(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagRemoveResponse:
        """
        Remove Tags from Limit

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return self._patch(
            f"/api/v1/limits/{limit_id}/tags/remove",
            body=maybe_transform({"limit_tags": limit_tags}, tag_remove_params.TagRemoveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagRemoveResponse,
        )


class AsyncTagsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return AsyncTagsResourceWithStreamingResponse(self)

    async def create(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagCreateResponse:
        """
        Add Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return await self._post(
            f"/api/v1/limits/{limit_id}/tags",
            body=await async_maybe_transform({"limit_tags": limit_tags}, tag_create_params.TagCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagCreateResponse,
        )

    async def update(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagUpdateResponse:
        """
        Update the Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return await self._put(
            f"/api/v1/limits/{limit_id}/tags",
            body=await async_maybe_transform({"limit_tags": limit_tags}, tag_update_params.TagUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagUpdateResponse,
        )

    async def list(
        self,
        limit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagListResponse:
        """
        Get Limit Tags

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return await self._get(
            f"/api/v1/limits/{limit_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )

    async def delete(
        self,
        limit_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagDeleteResponse:
        """
        Delete all Tags from Limit

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return await self._delete(
            f"/api/v1/limits/{limit_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagDeleteResponse,
        )

    async def remove(
        self,
        limit_id: str,
        *,
        limit_tags: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TagRemoveResponse:
        """
        Remove Tags from Limit

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not limit_id:
            raise ValueError(f"Expected a non-empty value for `limit_id` but received {limit_id!r}")
        return await self._patch(
            f"/api/v1/limits/{limit_id}/tags/remove",
            body=await async_maybe_transform({"limit_tags": limit_tags}, tag_remove_params.TagRemoveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagRemoveResponse,
        )


class TagsResourceWithRawResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.create = to_raw_response_wrapper(
            tags.create,
        )
        self.update = to_raw_response_wrapper(
            tags.update,
        )
        self.list = to_raw_response_wrapper(
            tags.list,
        )
        self.delete = to_raw_response_wrapper(
            tags.delete,
        )
        self.remove = to_raw_response_wrapper(
            tags.remove,
        )


class AsyncTagsResourceWithRawResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.create = async_to_raw_response_wrapper(
            tags.create,
        )
        self.update = async_to_raw_response_wrapper(
            tags.update,
        )
        self.list = async_to_raw_response_wrapper(
            tags.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tags.delete,
        )
        self.remove = async_to_raw_response_wrapper(
            tags.remove,
        )


class TagsResourceWithStreamingResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.create = to_streamed_response_wrapper(
            tags.create,
        )
        self.update = to_streamed_response_wrapper(
            tags.update,
        )
        self.list = to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = to_streamed_response_wrapper(
            tags.delete,
        )
        self.remove = to_streamed_response_wrapper(
            tags.remove,
        )


class AsyncTagsResourceWithStreamingResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.create = async_to_streamed_response_wrapper(
            tags.create,
        )
        self.update = async_to_streamed_response_wrapper(
            tags.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tags.delete,
        )
        self.remove = async_to_streamed_response_wrapper(
            tags.remove,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["LimitListParams"]


class LimitListParams(TypedDict, total=False):
    limit_name: str

    page_number: int

    page_size: int

    sort_ascending: bool

    sort_by: str

    tags: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import datetime

import httpx

from ..types import ingest_units_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    strip_not_given,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.ingest_response import IngestResponse
from ..types.ingest_event_param import IngestEventParam
from ..types.bulk_ingest_response import BulkIngestResponse
from ..types.pay_i_common_models_api_router_header_info_param import PayICommonModelsAPIRouterHeaderInfoParam

__all__ = ["IngestResource", "AsyncIngestResource"]


class IngestResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IngestResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return IngestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IngestResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return IngestResourceWithStreamingResponse(self)

    def bulk(
        self,
        *,
        events: Iterable[IngestEventParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BulkIngestResponse:
        """
        Bulk Ingest

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/ingest/bulk",
            body=maybe_transform(events, Iterable[IngestEventParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkIngestResponse,
        )

    def units(
        self,
        *,
        category: str,
        resource: str,
        units: Dict[str, ingest_units_params.Units],
        end_to_end_latency_ms: Optional[int] | NotGiven = NOT_GIVEN,
        event_timestamp: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        experience_properties: Optional[Dict[str, str]] | NotGiven = NOT_GIVEN,
        http_status_code: Optional[int] | NotGiven = NOT_GIVEN,
        properties: Optional[Dict[str, str]] | NotGiven = NOT_GIVEN,
        provider_request_headers: Optional[Iterable[PayICommonModelsAPIRouterHeaderInfoParam]] | NotGiven = NOT_GIVEN,
        provider_request_json: Optional[str] | NotGiven = NOT_GIVEN,
        provider_response_headers: Optional[Iterable[PayICommonModelsAPIRouterHeaderInfoParam]] | NotGiven = NOT_GIVEN,
        provider_response_json: Union[str, List[str], None] | NotGiven = NOT_GIVEN,
        provider_uri: Optional[str] | NotGiven = NOT_GIVEN,
        time_to_first_token_ms: Optional[int] | NotGiven = NOT_GIVEN,
        limit_ids: Union[list[str], None] | NotGiven = NOT_GIVEN,
        request_tags: Union[list[str], None] | NotGiven = NOT_GIVEN,
        experience_id: Union[str, None] | NotGiven = NOT_GIVEN,
        experience_name: Union[str, None] | NotGiven = NOT_GIVEN,
        user_id: Union[str, None] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IngestResponse:
        """
        Ingest an Event

        Args:
          category (str): The name of the category

          resource (str): The name of the resource

          input (int): The number of input units

          output (int): The number of output units

          event_timestamp: (str, datetime, None): The timestamp of the event. Defaults to None.

          limit_ids (list[str], optional): The limit IDs to associate with the request. Defaults to None.

          request_tags (list[str], optional): The request tags to associate with the request. Defaults to None.

          experience_name (str, optional): The experience name

          experience_id (str, optional): The experience instance id

          user_id (str, optional): The user id

          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.

          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.

          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.

          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        valid_ids_str: str | NotGiven = NOT_GIVEN
        valid_tags_str: str | NotGiven = NOT_GIVEN

        if limit_ids is None or isinstance(limit_ids, NotGiven):
            valid_ids_str = NOT_GIVEN
        elif not isinstance(limit_ids, list):  # type: ignore
            raise TypeError("limit_ids must be a list")
        else:
            # Proceed with the list comprehension if limit_ids is not NotGiven
            valid_ids = [id.strip() for id in limit_ids if id.strip()]
            valid_ids_str = ",".join(valid_ids) if valid_ids else NOT_GIVEN

        if request_tags is None or isinstance(request_tags, NotGiven):
            valid_tags_str = NOT_GIVEN
        elif not isinstance(request_tags, list):  # type: ignore
            raise TypeError("request_tags must be a list")
        else:
            # Proceed with the list comprehension if request_tags is not NotGiven
            valid_tags = [tag.strip() for tag in request_tags if tag.strip()]
            valid_tags_str = ",".join(valid_tags) if valid_tags else NOT_GIVEN

        if experience_name is None or isinstance(experience_name, NotGiven):
            experience_name = NOT_GIVEN

        if experience_id is None or isinstance(experience_id, NotGiven):
            experience_id = NOT_GIVEN

        if user_id is None or isinstance(user_id, NotGiven):
            user_id = NOT_GIVEN

        extra_headers = {
            **strip_not_given(
                {
                    "xProxy-Limit-IDs": valid_ids_str,
                    "xProxy-Request-Tags": valid_tags_str,
                    "xProxy-Experience-Id": experience_id,
                    "xProxy-Experience-Name": experience_name,
                    "xProxy-User-ID": user_id,
                }
            ),
            **(extra_headers or {}),
        }
        return self._post(
            "/api/v1/ingest",
            body=maybe_transform(
                {
                    "category": category,
                    "resource": resource,
                    "units": units,
                    "end_to_end_latency_ms": end_to_end_latency_ms,
                    "event_timestamp": event_timestamp,
                    "experience_properties": experience_properties,
                    "http_status_code": http_status_code,
                    "properties": properties,
                    "provider_request_headers": provider_request_headers,
                    "provider_request_json": provider_request_json,
                    "provider_response_headers": provider_response_headers,
                    "provider_response_json": provider_response_json,
                    "provider_uri": provider_uri,
                    "time_to_first_token_ms": time_to_first_token_ms,
                },
                ingest_units_params.IngestUnitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IngestResponse,
        )


class AsyncIngestResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIngestResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIngestResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIngestResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return AsyncIngestResourceWithStreamingResponse(self)

    async def bulk(
        self,
        *,
        events: Iterable[IngestEventParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BulkIngestResponse:
        """
        Bulk Ingest

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/ingest/bulk",
            body=await async_maybe_transform(events, Iterable[IngestEventParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BulkIngestResponse,
        )

    async def units(
        self,
        *,
        category: str,
        resource: str,
        units: Dict[str, ingest_units_params.Units],
        end_to_end_latency_ms: Optional[int] | NotGiven = NOT_GIVEN,
        event_timestamp: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        experience_properties: Optional[Dict[str, str]] | NotGiven = NOT_GIVEN,
        http_status_code: Optional[int] | NotGiven = NOT_GIVEN,
        properties: Optional[Dict[str, str]] | NotGiven = NOT_GIVEN,
        provider_request_headers: Optional[Iterable[PayICommonModelsAPIRouterHeaderInfoParam]] | NotGiven = NOT_GIVEN,
        provider_request_json: Optional[str] | NotGiven = NOT_GIVEN,
        provider_response_headers: Optional[Iterable[PayICommonModelsAPIRouterHeaderInfoParam]] | NotGiven = NOT_GIVEN,
        provider_response_json: Union[str, List[str], None] | NotGiven = NOT_GIVEN,
        provider_uri: Optional[str] | NotGiven = NOT_GIVEN,
        time_to_first_token_ms: Optional[int] | NotGiven = NOT_GIVEN,
        limit_ids: Union[list[str], None] | NotGiven = NOT_GIVEN,
        request_tags: Union[list[str], None] | NotGiven = NOT_GIVEN,
        experience_name: Union[str, None] | NotGiven = NOT_GIVEN,
        experience_id: Union[str, None] | NotGiven = NOT_GIVEN,
        user_id: Union[str, None] | NotGiven = NOT_GIVEN,
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IngestResponse:
        """
        Ingest an Event

        Args:
          category (str): The name of the category

          resource (str): The name of the resource

          input (int): The number of input units

          output (int): The number of output units

          event_timestamp: (datetime, None): The timestamp of the event. Defaults to None.

          limit_ids (list[str], optional): The limit IDs to associate with the request. Defaults to None.

          request_tags (list[str], optional): The request tags to associate with the request. Defaults to None.

          experience_name (str, optional): The experience name

          experience_id (str, optional): The experience instance id

          user_id (str, optional): The user id

          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.

          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.

          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.

          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        valid_ids_str: str | NotGiven = NOT_GIVEN
        valid_tags_str: str | NotGiven = NOT_GIVEN

        if limit_ids is None or isinstance(limit_ids, NotGiven):
            valid_ids_str = NOT_GIVEN
        elif not isinstance(limit_ids, list):  # type: ignore
            raise TypeError("limit_ids must be a list")
        else:
            # Proceed with the list comprehension if limit_ids is not NotGiven
            valid_ids = [id.strip() for id in limit_ids if id.strip()]
            valid_ids_str = ",".join(valid_ids) if valid_ids else NOT_GIVEN

        if request_tags is None or isinstance(request_tags, NotGiven):
            valid_tags_str = NOT_GIVEN
        elif not isinstance(request_tags, list):  # type: ignore
            raise TypeError("request_tags must be a list")
        else:
            # Proceed with the list comprehension if request_tags is not NotGiven
            valid_tags = [tag.strip() for tag in request_tags if tag.strip()]
            valid_tags_str = ",".join(valid_tags) if valid_tags else NOT_GIVEN

        if experience_name is None or isinstance(experience_name, NotGiven):
            experience_name = NOT_GIVEN

        if experience_id is None or isinstance(experience_id, NotGiven):
            experience_id = NOT_GIVEN

        if user_id is None or isinstance(user_id, NotGiven):
            user_id = NOT_GIVEN

        extra_headers = {
            **strip_not_given(
                {
                    "xProxy-Limit-IDs": valid_ids_str,
                    "xProxy-Request-Tags": valid_tags_str,
                    "xProxy-Experience-Name": experience_name,
                    "xProxy-Experience-Id": experience_id,
                    "xProxy-User-ID": user_id,
                }
            ),
            **(extra_headers or {}),
        }
        return await self._post(
            "/api/v1/ingest",
            body=await async_maybe_transform(
                {
                    "category": category,
                    "resource": resource,
                    "units": units,
                    "end_to_end_latency_ms": end_to_end_latency_ms,
                    "event_timestamp": event_timestamp,
                    "experience_properties": experience_properties,
                    "http_status_code": http_status_code,
                    "properties": properties,
                    "provider_request_headers": provider_request_headers,
                    "provider_request_json": provider_request_json,
                    "provider_response_headers": provider_response_headers,
                    "provider_response_json": provider_response_json,
                    "provider_uri": provider_uri,
                    "time_to_first_token_ms": time_to_first_token_ms,
                },
                ingest_units_params.IngestUnitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IngestResponse,
        )


class IngestResourceWithRawResponse:
    def __init__(self, ingest: IngestResource) -> None:
        self._ingest = ingest

        self.bulk = to_raw_response_wrapper(
            ingest.bulk,
        )
        self.units = to_raw_response_wrapper(
            ingest.units,
        )


class AsyncIngestResourceWithRawResponse:
    def __init__(self, ingest: AsyncIngestResource) -> None:
        self._ingest = ingest

        self.bulk = async_to_raw_response_wrapper(
            ingest.bulk,
        )
        self.units = async_to_raw_response_wrapper(
            ingest.units,
        )


class IngestResourceWithStreamingResponse:
    def __init__(self, ingest: IngestResource) -> None:
        self._ingest = ingest

        self.bulk = to_streamed_response_wrapper(
            ingest.bulk,
        )
        self.units = to_streamed_response_wrapper(
            ingest.units,
        )


class AsyncIngestResourceWithStreamingResponse:
    def __init__(self, ingest: AsyncIngestResource) -> None:
        self._ingest = ingest

        self.bulk = async_to_streamed_response_wrapper(
            ingest.bulk,
        )
        self.units = async_to_streamed_response_wrapper(
            ingest.units,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from payi.types import CategoryResourceResponse
from payi._utils import parse_datetime
from tests.utils import assert_matches_type
from payi.types.categories import ResourceListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResources:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Payi) -> None:
        resource = client.categories.resources.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Payi) -> None:
        resource = client.categories.resources.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
            max_input_units=0,
            max_output_units=0,
            max_total_units=0,
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Payi) -> None:
        response = client.categories.resources.with_raw_response.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Payi) -> None:
        with client.categories.resources.with_streaming_response.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            client.categories.resources.with_raw_response.create(
                resource="resource",
                category="",
                units={
                    "foo": {
                        "input_price": 0,
                        "output_price": 0,
                    }
                },
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            client.categories.resources.with_raw_response.create(
                resource="",
                category="category",
                units={
                    "foo": {
                        "input_price": 0,
                        "output_price": 0,
                    }
                },
            )

    @parametrize
    def test_method_retrieve(self, client: Payi) -> None:
        resource = client.categories.resources.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Payi) -> None:
        response = client.categories.resources.with_raw_response.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Payi) -> None:
        with client.categories.resources.with_streaming_response.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            client.categories.resources.with_raw_response.retrieve(
                resource_id="resource_id",
                category="",
                resource="resource",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            client.categories.resources.with_raw_response.retrieve(
                resource_id="resource_id",
                category="category",
                resource="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            client.categories.resources.with_raw_response.retrieve(
                resource_id="",
                category="category",
                resource="resource",
            )

    @parametrize
    def test_method_list(self, client: Payi) -> None:
        resource = client.categories.resources.list(
            resource="resource",
            category="category",
        )
        assert_matches_type(ResourceListResponse, resource, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Payi) -> None:
        response = client.categories.resources.with_raw_response.list(
            resource="resource",
            category="category",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = response.parse()
        assert_matches_type(ResourceListResponse, resource, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Payi) -> None:
        with client.categories.resources.with_streaming_response.list(
            resource="resource",
            category="category",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = response.parse()
            assert_matches_type(ResourceListResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            client.categories.resources.with_raw_response.list(
                resource="resource",
                category="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            client.categories.resources.with_raw_response.list(
                resource="",
                category="category",
            )

    @parametrize
    def test_method_delete(self, client: Payi) -> None:
        resource = client.categories.resources.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Payi) -> None:
        response = client.categories.resources.with_raw_response.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Payi) -> None:
        with client.categories.resources.with_streaming_response.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            client.categories.resources.with_raw_response.delete(
                resource_id="resource_id",
                category="",
                resource="resource",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            client.categories.resources.with_raw_response.delete(
                resource_id="resource_id",
                category="category",
                resource="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            client.categories.resources.with_raw_response.delete(
                resource_id="",
                category="category",
                resource="resource",
            )


class TestAsyncResources:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPayi) -> None:
        resource = await async_client.categories.resources.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPayi) -> None:
        resource = await async_client.categories.resources.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
            max_input_units=0,
            max_output_units=0,
            max_total_units=0,
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPayi) -> None:
        response = await async_client.categories.resources.with_raw_response.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = await response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPayi) -> None:
        async with async_client.categories.resources.with_streaming_response.create(
            resource="resource",
            category="category",
            units={
                "foo": {
                    "input_price": 0,
                    "output_price": 0,
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = await response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            await async_client.categories.resources.with_raw_response.create(
                resource="resource",
                category="",
                units={
                    "foo": {
                        "input_price": 0,
                        "output_price": 0,
                    }
                },
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            await async_client.categories.resources.with_raw_response.create(
                resource="",
                category="category",
                units={
                    "foo": {
                        "input_price": 0,
                        "output_price": 0,
                    }
                },
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPayi) -> None:
        resource = await async_client.categories.resources.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPayi) -> None:
        response = await async_client.categories.resources.with_raw_response.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = await response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPayi) -> None:
        async with async_client.categories.resources.with_streaming_response.retrieve(
            resource_id="resource_id",
            category="category",
            resource="resource",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = await response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            await async_client.categories.resources.with_raw_response.retrieve(
                resource_id="resource_id",
                category="",
                resource="resource",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            await async_client.categories.resources.with_raw_response.retrieve(
                resource_id="resource_id",
                category="category",
                resource="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            await async_client.categories.resources.with_raw_response.retrieve(
                resource_id="",
                category="category",
                resource="resource",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPayi) -> None:
        resource = await async_client.categories.resources.list(
            resource="resource",
            category="category",
        )
        assert_matches_type(ResourceListResponse, resource, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPayi) -> None:
        response = await async_client.categories.resources.with_raw_response.list(
            resource="resource",
            category="category",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = await response.parse()
        assert_matches_type(ResourceListResponse, resource, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPayi) -> None:
        async with async_client.categories.resources.with_streaming_response.list(
            resource="resource",
            category="category",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = await response.parse()
            assert_matches_type(ResourceListResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            await async_client.categories.resources.with_raw_response.list(
                resource="resource",
                category="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            await async_client.categories.resources.with_raw_response.list(
                resource="",
                category="category",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncPayi) -> None:
        resource = await async_client.categories.resources.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncPayi) -> None:
        response = await async_client.categories.resources.with_raw_response.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        resource = await response.parse()
        assert_matches_type(CategoryResourceResponse, resource, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncPayi) -> None:
        async with async_client.categories.resources.with_streaming_response.delete(
            resource_id="resource_id",
            category="category",
            resource="resource",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            resource = await response.parse()
            assert_matches_type(CategoryResourceResponse, resource, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `category` but received ''"):
            await async_client.categories.resources.with_raw_response.delete(
                resource_id="resource_id",
                category="",
                resource="resource",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource` but received ''"):
            await async_client.categories.resources.with_raw_response.delete(
                resource_id="resource_id",
                category="category",
                resource="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            await async_client.categories.resources.with_raw_response.delete(
                resource_id="",
                category="category",
                resource="resource",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .limit_tags import LimitTags

__all__ = ["TagListResponse"]

TagListResponse: TypeAlias = List[LimitTags]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from payi.types import IngestResponse, BulkIngestResponse
from payi._utils import parse_datetime
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIngest:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_bulk(self, client: Payi) -> None:
        ingest = client.ingest.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        )
        assert_matches_type(BulkIngestResponse, ingest, path=["response"])

    @parametrize
    def test_raw_response_bulk(self, client: Payi) -> None:
        response = client.ingest.with_raw_response.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = response.parse()
        assert_matches_type(BulkIngestResponse, ingest, path=["response"])

    @parametrize
    def test_streaming_response_bulk(self, client: Payi) -> None:
        with client.ingest.with_streaming_response.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = response.parse()
            assert_matches_type(BulkIngestResponse, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_units(self, client: Payi) -> None:
        ingest = client.ingest.units(
            category="x",
            resource="x",
            units={"foo": {}},
        )
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    def test_method_units_with_all_params(self, client: Payi) -> None:
        ingest = client.ingest.units(
            category="x",
            resource="x",
            units={
                "foo": {
                    "input": 0,
                    "output": 0,
                }
            },
            end_to_end_latency_ms=0,
            event_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            experience_properties={"foo": "string"},
            http_status_code=0,
            properties={"foo": "string"},
            provider_request_headers=[
                {
                    "name": "x",
                    "value": "value",
                }
            ],
            provider_request_json="provider_request_json",
            provider_response_headers=[
                {
                    "name": "x",
                    "value": "value",
                }
            ],
            provider_response_json="string",
            provider_uri="provider_uri",
            time_to_first_token_ms=0,
            limit_ids=["limitId1", "limitId_2"],
            request_tags=["requestTag1", "request_tag_2"],
            experience_name="experience_name",
            experience_id="experience_id",
            user_id="UserName123",
        )
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    def test_raw_response_units(self, client: Payi) -> None:
        response = client.ingest.with_raw_response.units(
            category="x",
            resource="x",
            units={"foo": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = response.parse()
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    def test_streaming_response_units(self, client: Payi) -> None:
        with client.ingest.with_streaming_response.units(
            category="x",
            resource="x",
            units={"foo": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = response.parse()
            assert_matches_type(IngestResponse, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncIngest:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_bulk(self, async_client: AsyncPayi) -> None:
        ingest = await async_client.ingest.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        )
        assert_matches_type(BulkIngestResponse, ingest, path=["response"])

    @parametrize
    async def test_raw_response_bulk(self, async_client: AsyncPayi) -> None:
        response = await async_client.ingest.with_raw_response.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = await response.parse()
        assert_matches_type(BulkIngestResponse, ingest, path=["response"])

    @parametrize
    async def test_streaming_response_bulk(self, async_client: AsyncPayi) -> None:
        async with async_client.ingest.with_streaming_response.bulk(
            events=[
                {
                    "category": "x",
                    "resource": "x",
                    "units": {"foo": {}},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = await response.parse()
            assert_matches_type(BulkIngestResponse, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_units(self, async_client: AsyncPayi) -> None:
        ingest = await async_client.ingest.units(
            category="x",
            resource="x",
            units={"foo": {}},
        )
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    async def test_method_units_with_all_params(self, async_client: AsyncPayi) -> None:
        ingest = await async_client.ingest.units(
            category="x",
            resource="x",
            units={
                "foo": {
                    "input": 0,
                    "output": 0,
                }
            },
            end_to_end_latency_ms=0,
            event_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            experience_properties={"foo": "string"},
            http_status_code=0,
            properties={"foo": "string"},
            provider_request_headers=[
                {
                    "name": "x",
                    "value": "value",
                }
            ],
            provider_request_json="provider_request_json",
            provider_response_headers=[
                {
                    "name": "x",
                    "value": "value",
                }
            ],
            provider_response_json="string",
            provider_uri="provider_uri",
            time_to_first_token_ms=0,
            limit_ids=["limitId1", "limitId_2"],
            request_tags=["requestTag1", "request_tag_2"],
            experience_name="experience_name",
            experience_id="experience_id",
            user_id="UserName123",
        )
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    async def test_raw_response_units(self, async_client: AsyncPayi) -> None:
        response = await async_client.ingest.with_raw_response.units(
            category="x",
            resource="x",
            units={"foo": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ingest = await response.parse()
        assert_matches_type(IngestResponse, ingest, path=["response"])

    @parametrize
    async def test_streaming_response_units(self, async_client: AsyncPayi) -> None:
        async with async_client.ingest.with_streaming_response.units(
            category="x",
            resource="x",
            units={"foo": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ingest = await response.parse()
            assert_matches_type(IngestResponse, ingest, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .billing_model import BillingModel

__all__ = ["BillingModelListResponse"]

BillingModelListResponse: TypeAlias = List[BillingModel]



from typing import Dict, List, Union


class PayiHeaderNames:
    limit_ids:str  = "xProxy-Limit-IDs"
    request_tags:str = "xProxy-Request-Tags"
    experience_id:str = "xProxy-Experience-ID"
    experience_name:str = "xProxy-Experience-Name"
    user_id:str = "xProxy-User-ID"
    route_as_resource:str = "xProxy-RouteAs-Resource"
    provider_base_uri = "xProxy-Provider-BaseUri"

def create_limit_header_from_ids(limit_ids: List[str]) -> Dict[str, str]:
    if not isinstance(limit_ids, list):  # type: ignore
        raise TypeError("limit_ids must be a list")

    valid_ids = [id.strip() for id in limit_ids if isinstance(id, str) and id.strip()]  # type: ignore

    return { PayiHeaderNames.limit_ids: ",".join(valid_ids) } if valid_ids else {}


def create_request_header_from_tags(request_tags: List[str]) -> Dict[str, str]:
    if not isinstance(request_tags, list):  # type: ignore
        raise TypeError("request_tags must be a list")

    valid_tags = [tag.strip() for tag in request_tags if isinstance(tag, str) and tag.strip()]  # type: ignore

    return { PayiHeaderNames.request_tags: ",".join(valid_tags) } if valid_tags else {}


def create_headers(
    limit_ids: Union[List[str], None] = None,
    request_tags: Union[List[str], None] = None,
    user_id: Union[str, None] = None,
    experience_id: Union[str, None] = None,
    experience_name: Union[str, None] = None,
) -> Dict[str, str]:
    headers: Dict[str, str] = {}

    if limit_ids:
        headers.update(create_limit_header_from_ids(limit_ids))
    if request_tags:
        headers.update(create_request_header_from_tags(request_tags))
    if user_id:
        headers.update({ PayiHeaderNames.user_id: user_id})
    if experience_id:
        headers.update({ PayiHeaderNames.experience_id: experience_id})
    if experience_name:
        headers.update({ PayiHeaderNames.experience_name: experience_name})

    return headers



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PriceModifierUpdateParams"]


class PriceModifierUpdateParams(TypedDict, total=False):
    billing_model_id: Required[str]

    category: Required[str]

    price_modifier: Required[float]

    resource: Required[str]
